/* All Rights reserved */

#include <AppKit/AppKit.h>
#include "MyController.h"
#import "MyView.h"

@interface ThreadHelper : NSObject
{
  NSImage *image;
  NSBitmapImageRep *imageRep;
}
@end

@implementation ThreadHelper

- (void)updateNoscaleImage: (id)aController
{
  [aController updateNoscaleThreadImage: image];
}

- (void)updateScaleImage: (id)aController
{
  [aController updateScaleThreadImage: image];
}

- (void)doNoscaleImage: (id)aController
{
  NSAutoreleasePool *pool = [NSAutoreleasePool new];

  NSLog(@"doNoscaleImage:");

  NSRect imRect = NSMakeRect(0, 0, 80, 80);
  image = [[NSImage alloc] initWithSize:imRect.size];
  imageRep = [[NSBitmapImageRep alloc]
	  initWithBitmapDataPlanes:NULL
	  pixelsWide:imRect.size.width
	  pixelsHigh:imRect.size.height
	  bitsPerSample:8
	  samplesPerPixel:3
	  hasAlpha:NO
	  isPlanar:NO
	  colorSpaceName:NSDeviceRGBColorSpace
	  bytesPerRow:0
	  bitsPerPixel:0];
  [image addRepresentation:imageRep];

  // draw some random dots
  [image lockFocus];

  [[NSColor blackColor] set];
  NSRectFill(imRect);

  int i;
  for (i = 0; i < 100; ++i) {
    int x = 80 * ((double)rand() / (double)RAND_MAX);
    int y = 80 * ((double)rand() / (double)RAND_MAX);
    NSRect dotRect = NSMakeRect(x,y,1,1);
    [[NSColor blueColor] set];
    NSRectFill(dotRect);
  }

  [image unlockFocus];

  // get image from file
  //image = [[NSImage alloc] initWithContentsOfFile: @"heatbugs.tiff"];

  // signal the main thread to perform the image display
  [self performSelectorOnMainThread: @selector(updateNoscaleImage:)
	withObject: aController
	waitUntilDone: YES];

  [pool release];
}

- (void)doScaleImage: (id)aController
{
  NSAutoreleasePool *pool = [NSAutoreleasePool new];

  NSLog(@"doScaleImage:");

  NSRect imRect = NSMakeRect(0, 0, 80, 80);
  image = [[NSImage alloc] initWithSize:imRect.size];
  imageRep = [[NSBitmapImageRep alloc]
	  initWithBitmapDataPlanes:NULL
	  pixelsWide:imRect.size.width
	  pixelsHigh:imRect.size.height
	  bitsPerSample:8
	  samplesPerPixel:3
	  hasAlpha:NO
	  isPlanar:NO
	  colorSpaceName:NSDeviceRGBColorSpace
	  bytesPerRow:0
	  bitsPerPixel:0];
  [image addRepresentation:imageRep];

  // draw some random dots
  [image lockFocus];

  [[NSColor blackColor] set];
  NSRectFill(imRect);

  int i;
  for (i = 0; i < 100; ++i) {
    int x = 80 * ((double)rand() / (double)RAND_MAX);
    int y = 80 * ((double)rand() / (double)RAND_MAX);
    NSRect dotRect = NSMakeRect(x,y,1,1);
    [[NSColor blueColor] set];
    NSRectFill(dotRect);
  }

  [image unlockFocus];

  // get image from file
  //image = [[NSImage alloc] initWithContentsOfFile: @"heatbugs.tiff"];

  // signal the main thread to perform the image display
  [self performSelectorOnMainThread: @selector(updateScaleImage:)
	withObject: aController
	waitUntilDone: YES];

  [pool release];
}

@end


@implementation MyController

- (void) awakeFromNib
{
  [fileImage setBehaviorCode: 1];
  [noscaleImage setBehaviorCode: 2];
  [scaleImage setBehaviorCode: 3];
}

- (void) drawFileImage: (id)sender
{
  NSLog(@"drawFileImage:");
  NSImage *image = [[NSImage alloc] initWithContentsOfFile: @"heatbugs.tiff"];

  [fileImageOutside lockFocus];
  [image drawInRect: [fileImageOutside bounds]
	 fromRect: NSZeroRect operation: NSCompositeSourceAtop fraction: 1.0];
  [fileImageOutside unlockFocus];
}


- (void) drawNoscaleImage: (id)sender
{
  NSLog(@"drawNoscaleImage:");

  NSRect imRect = NSMakeRect(0, 0, 80, 80);
  NSImage *image = [[NSImage alloc] initWithSize:imRect.size];
  NSBitmapImageRep *imageRep = [[NSBitmapImageRep alloc]
	  initWithBitmapDataPlanes:NULL
	  pixelsWide:imRect.size.width
	  pixelsHigh:imRect.size.height
	  bitsPerSample:8
	  samplesPerPixel:3
	  hasAlpha:NO
	  isPlanar:NO
	  colorSpaceName:NSDeviceRGBColorSpace
	  bytesPerRow:0
	  bitsPerPixel:0];
  [image addRepresentation:imageRep];

  // draw some random dots
  [image lockFocus];

  [[NSColor blackColor] set];
  NSRectFill(imRect);

  int i;
  for (i = 0; i < 100; ++i) {
    int x = 80 * ((double)rand() / (double)RAND_MAX);
    int y = 80 * ((double)rand() / (double)RAND_MAX);
    NSRect dotRect = NSMakeRect(x,y,1,1);
    [[NSColor greenColor] set];
    NSRectFill(dotRect);
  }

  [image unlockFocus];

  [noscaleImageOutside lockFocus];
  [image drawInRect: imRect fromRect: NSZeroRect operation: NSCompositeSourceAtop fraction: 1.0];
  [noscaleImageOutside unlockFocus];
}


- (void) drawScaleImage: (id)sender
{
  NSLog(@"drawScaleImage:");

  NSRect imRect = NSMakeRect(0, 0, 80, 80);
  NSImage *image = [[NSImage alloc] initWithSize:imRect.size];
  NSBitmapImageRep *imageRep = [[NSBitmapImageRep alloc]
	  initWithBitmapDataPlanes:NULL
	  pixelsWide:imRect.size.width
	  pixelsHigh:imRect.size.height
	  bitsPerSample:8
	  samplesPerPixel:3
	  hasAlpha:NO
	  isPlanar:NO
	  colorSpaceName:NSDeviceRGBColorSpace
	  bytesPerRow:0
	  bitsPerPixel:0];
  [image addRepresentation:imageRep];

  // draw some random dots
  [image lockFocus];

  [[NSColor blackColor] set];
  NSRectFill(imRect);

  int i;
  for (i = 0; i < 100; ++i) {
    int x = 80 * ((double)rand() / (double)RAND_MAX);
    int y = 80 * ((double)rand() / (double)RAND_MAX);
    NSRect dotRect = NSMakeRect(x,y,1,1);
    [[NSColor greenColor] set];
    NSRectFill(dotRect);
  }

  [image unlockFocus];

  // scale to view bounds
  [scaleImageOutside lockFocus];
  [image drawInRect: [scaleImageOutside bounds]
	 fromRect: NSZeroRect operation: NSCompositeSourceAtop fraction: 1.0];
  [scaleImageOutside unlockFocus];
}

- (void) drawScaleThreadImage: (id)sender
{
  ThreadHelper *aHelper = [ThreadHelper new];

  [NSThread detachNewThreadSelector: @selector(doScaleImage:)
	    toTarget: aHelper withObject: self];
}

- (void) drawNoscaleThreadImage: (id)sender
{
  ThreadHelper *aHelper = [ThreadHelper new];

  [NSThread detachNewThreadSelector: @selector(doNoscaleImage:)
	    toTarget: aHelper withObject: self];
}

- (void)updateNoscaleThreadImage: (NSImage *)anImage
{
  NSLog(@"updateNoscaleThreadImage:");

  NSSize aSize = [anImage size];
  NSRect aRect = NSMakeRect(0,0,aSize.width, aSize.height);

  printf("aRect: %f %f %f %f\n", aRect.origin.x, aRect.origin.y,
	 aRect.size.width, aRect.size.height);

  [noscaleThreadImage lockFocus];
  [anImage drawInRect: aRect
	 fromRect: NSZeroRect operation: NSCompositeSourceAtop fraction: 1.0];
  [noscaleThreadImage unlockFocus];
}

- (void)updateScaleThreadImage: (NSImage *)anImage
{
  NSLog(@"updateScaleThreadImage:");
  [scaleThreadImage lockFocus];
  [anImage drawInRect: [scaleThreadImage bounds]
	 fromRect: NSZeroRect operation: NSCompositeSourceAtop fraction: 1.0];
  [scaleThreadImage unlockFocus];
}

@end
